/*
 * checkerberry test center - https://www.checkerberry.de
 *
 * Copyright (c) 2010 ConceptPeople consulting gmbh
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/software/page/eupl5
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.conceptpeople.checkerberry.db.example.core.dump;

import java.io.File;
import java.io.IOException;
import java.nio.charset.Charset;
import java.nio.file.Files;
import java.nio.file.Paths;

import org.junit.Assert;
import org.junit.Test;

import de.conceptpeople.checkerberry.db.core.dump.ForeignKey;
import de.conceptpeople.checkerberry.db.core.dump.Recipe;
import de.conceptpeople.checkerberry.db.core.dump.Selection;
import de.conceptpeople.checkerberry.db.core.dump.TableConstraint;
import de.conceptpeople.checkerberry.db.core.dump.TableConstraint.Type;
import de.conceptpeople.checkerberry.db.core.test.DbTestHandler;
import de.conceptpeople.checkerberry.db.example.AbstractExampleTestCase;

public class DataSetExtractorTest extends AbstractExampleTestCase {

    @Test
    public void dumpUserAndDishTableWithSelectedUser() throws IOException {
        String tempFileName = createTempDumpFile("dump-test");

        Recipe recipe = Recipe.with(Selection.table("USERS")
                    .where(TableConstraint.key("ID").eq().to(1))
                    .build())
                .build();

        DbTestHandler testHandler = getEnvironment().getTestHandler();

        testHandler.dumpTables(tempFileName, recipe);

        String result = new String(Files.readAllBytes(Paths.get(tempFileName)), Charset.forName("UTF-8"));

        String match="<?xml version='1.0' encoding='UTF-8'?>\n"+
            "<!DOCTYPE dataset PUBLIC \"-//ConceptPeople//DTD example-db 1.0//EN\" \"de/conceptpeople/checkerberry/db/example/example-db.dtd\">\n"+
            "<dataset>\n"+
            "  <USERS ID=\"1\" NAME=\"Homer\" SURNAME=\"Simpson\" BIRTHDATE=\"1946-09-16\" WEIGHT=\"80.0\"/>\n"+
            "  <USERDISHREL ID=\"1\" DISH_ID=\"1\"/>\n"+
            "  <DISH DISH_ID=\"1\" DESCRIPTION=\"Donuts\"/>\n"+
            "</dataset>\n";

        Assert.assertEquals("dump matches",match,result);
    }

    @Test
    public void dumpUsersAndDishtableWithSelectedUserAndDish() throws IOException {
        String tempFileName = createTempDumpFile("dump-test");

        Recipe recipe = Recipe.with(Selection.table("USERS")
                    .where(TableConstraint.key("ID").eq().to(1))
                    .build())
                .and(Selection.table("DISH")
                    .where(TableConstraint.key("DISH_ID").eq().to(1))
                    .build())
                .build();

        DbTestHandler testHandler = getEnvironment().getTestHandler();

        testHandler.dumpTables(tempFileName, recipe);

        String result = new String(Files.readAllBytes(Paths.get(tempFileName)), Charset.forName("UTF-8"));

        String match="<?xml version='1.0' encoding='UTF-8'?>\n"+
            "<!DOCTYPE dataset PUBLIC \"-//ConceptPeople//DTD example-db 1.0//EN\" \"de/conceptpeople/checkerberry/db/example/example-db.dtd\">\n"+
            "<dataset>\n"+
            "  <USERS ID=\"1\" NAME=\"Homer\" SURNAME=\"Simpson\" BIRTHDATE=\"1946-09-16\" WEIGHT=\"80.0\"/>\n"+
            "  <USERS ID=\"2\" NAME=\"Marge\" SURNAME=\"Simpson\" BIRTHDATE=\"1950-09-16\" WEIGHT=\"60.0\"/>\n"+
            "  <USERDISHREL ID=\"1\" DISH_ID=\"1\"/>\n"+
            "  <USERDISHREL ID=\"2\" DISH_ID=\"1\"/>\n"+
            "  <DISH DISH_ID=\"1\" DESCRIPTION=\"Donuts\"/>\n"+
            "</dataset>\n";

        Assert.assertEquals("dump matches",match,result);
    }
    /**
     * UseCase für eine Selektion mit mehreren Einschränkungen
     */
    public void useCaseAddMultipleConstraints() {
        Recipe recipe = Recipe.with(Selection.table("USERS")
                .where(TableConstraint.key("ID").type(Type.GT).to(1))
                .where(TableConstraint.key("ID").type(Type.LT).to(10))
                .build())
            .build();
        Assert.assertNotNull(recipe);
    }

    /**
     * UseCase mit zusätzlicher ForeignKey-Beziehung.
     */
    public void useCaseAddCustomForeignKeys() {
        Recipe recipe = Recipe.with(Selection.table("USERS")
                .where(TableConstraint.key("ID").eq().to(1))
                .build())
            .foreignKey(ForeignKey.from("USERDISHREL", "DISH_ID").to("DISH", "DISH_ID"))
            .build();
        Assert.assertNotNull(recipe);
    }

    /**
     * UseCase mit zusätzlicher PrimaryKey-Definition.
     */
    public void useCaseAddCustomPrimaryKeys() {
        Recipe recipe = Recipe.with(Selection.table("USERS")
                .where(TableConstraint.key("ID").eq().to(1))
                .build())
            .primaryKey("USERDISHREL", "ID")
            .build();
        Assert.assertNotNull(recipe);
    }

    /**
     * UseCase mit zusätzlicher Selektion.
     */
    public void secondSelection() {
        Recipe recipe = Recipe.with(Selection.table("USERS")
                .where(TableConstraint.key("ID").eq().to(1))
                .build())
            .and(Selection.table("LOCATIONS")
                .where(TableConstraint.key("CityCode").eq().to("HH"))
                .build())
            .build();
        Assert.assertNotNull(recipe);
    }

    private String createTempDumpFile(String name) throws IOException {
        File dumpFile = File.createTempFile(name, ".xml");
        // Diese Zeile auskommentieren, um einen Blick in die Datei zu werfen.
        dumpFile.deleteOnExit();
        return dumpFile.getAbsolutePath();
    }
}
